"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.beginWork = void 0;
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const regexDefinition_1 = require("./regexDefinition");
const codeAnalyze_1 = require("./codeAnalyze");
const progress_bar_1 = require("./commandLineTools/progress-bar");
const Output = [];
let targetExtname;
let Callback;
let hasRegisterHandler;
// Used to show process bar
let hasProcessBar;
let taskCompleted;
let taskTotal = 0;
const taskLists = [];
// Input function
function beginWork(filePath, fileExtname, callback) {
    fileExtname && setTargetExtname(fileExtname);
    callback && setCallback(callback);
    if (checkIsDir(filePath)) {
        handleDir(filePath);
    }
    else {
        handleFile(filePath);
    }
    // Display a progress bar
    initProgressBar();
    ensureWorkIsDone();
}
exports.beginWork = beginWork;
function setTargetExtname(extname) {
    targetExtname = extname;
}
function setCallback(callback) {
    Callback = callback;
}
function checkIsDir(filePath) {
    return filePath.endsWith('/') || filePath.endsWith('\\');
}
function handleDir(filePath) {
    // Read dir
    const pathList = fs_1.default.readdirSync(path_1.default.resolve(filePath), { withFileTypes: true });
    // Walk the file list
    pathList.forEach(path => {
        if (path.isFile()) {
            handleFile(`${filePath}${path.name}`);
        }
        else {
            // Recursion. If it is a folder, continue to walk the files under the folder
            beginWork(`${filePath}${path.name}/`);
        }
    });
}
function handleFile(filePath) {
    if (!(targetExtname === null || targetExtname === void 0 ? void 0 : targetExtname.length) ||
        targetExtname.includes(path_1.default.extname(filePath))) {
        // Register a new work
        readFile(filePath);
    }
}
// Register new work
function readFile(filedir) {
    var _a;
    return __awaiter(this, void 0, void 0, function* () {
        const task = fs_1.default.promises.readFile(filedir, 'utf-8');
        // push it into queue
        taskLists.push(task);
        taskTotal++;
        const text = yield task;
        const regexResult = (_a = getCaptures(text, (0, regexDefinition_1.getRegex)())) !== null && _a !== void 0 ? _a : [];
        for (const data of regexResult) {
            // Use AST to analyze code
            const astReult = (0, codeAnalyze_1.analyzeCodeByAst)(data[0]);
            const codeInfoList = astReult.map(el => (Object.assign({ fileName: filedir }, el)));
            // collect the result in Output
            Output.push(...codeInfoList);
        }
    });
}
function ensureWorkIsDone() {
    if (hasRegisterHandler) {
        // If already register a handler, just return
        return;
    }
    hasRegisterHandler = true;
    Promise.all(taskLists).then(() => {
        if (taskCompleted < taskTotal) {
            // If the process hasn't be 100%, make it be
            taskCompleted = taskTotal;
        }
        // Make sure the bar goes to 100% before we run callback
        setTimeout(() => {
            if (!Callback) {
                console.log('Precession is done. But no callback function is detected.');
                return;
            }
            Callback(Output);
        }, 501);
    });
}
// Run RegExp.exec in string
function getCaptures(text, regex) {
    const regexResult = [];
    let match;
    while ((match = regex.exec(text)) !== null) {
        regexResult.push(match);
    }
    return regexResult;
}
function initProgressBar() {
    if (hasProcessBar) {
        return;
    }
    hasProcessBar = true;
    taskCompleted = 0;
    const pb = new progress_bar_1.ProgressBar('tracking', 30);
    function downloading() {
        if (taskCompleted <= taskTotal) {
            pb.render({ completed: taskCompleted, total: taskTotal });
            taskCompleted++;
            setTimeout(function () {
                downloading();
            }, 500);
        }
    }
    downloading();
}
